//+------------------------------------------------------------------+
//|                                         LevelStop-Reverse EA.mq4 |
//|                                                      Version 1.3 |
//|                                     Copyright  2010, stevegee58 |
//|                                             stevegee58@gmail.com |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+

/*

This is an EA that uses the LevelStop-Reverse indicator written by Bruce Hellstrom.
It implements the strictest interpretation of the Level Stop-Reverse system rules as specified
by the original VT Systems scripts.  The original VT Systems settings have been captured as
defaults in this EA (ATR Period and ATR Smoothing are 14 and ATR Multiplier is 2.824.

This strict interpretation means that the EA only opens a trade when the trailing stop flips over
to the other side of price.  The trailing stop on the currently forming bar may flip back and forth
in real time, but this is ignored.  The EA only opens the trade on the open of the bar after the
"flipping over" bar closes.  Obviously this could be improved upon, but my goal was to implement
what is best documented.

The boolean parameter DumbTradeRules is meant to trade according to this strict interpretation.  Currently
only the DumbTradeRules=true is implemented.  A future version will introduce an improved interpretation
with filtering to improve the system's performance.

This EA appears to have positive expectancy on EURUSD H1 with the default settings.

Revision History

Version 1.1
* Removed some superfluous Print statements.
* Corrected a comment.
* Fixed a bug where the "no signal" condition was not being detected.

Version 1.2
* Added time filter.  New trades are initiated only during the specified time window.

Version 1.3
* Changed name of indicator from LevelStop-Reverse.mq4 to LevelStop-Reverse2.0.mq4 since
  there are several versions floating around the interwebs.
* Added parameter to turn on/off the time filter.

*/

extern bool DumbTradeRules = true;
extern double Lots = 0.01;
extern bool UseATRMode = true;
extern int NonATRStopPips = 40;
extern int ATRPeriod = 14;
extern double ATRMultiplier = 2.824;
extern int ATRSmoothing = 14;
extern color UpArrowColor = DodgerBlue;
extern color DnArrowColor = OrangeRed;
extern int ArrowDistance = 0;
extern bool AlertSound = true;
extern bool AlertMail = false;
extern bool ShowComment = true;
extern int FromHourTrade = 10;
extern int ToHourTrade = 18;
extern bool UseHourTrade = True;

#define SIGNAL_NONE 0
#define SIGNAL_BUY   1
#define SIGNAL_SELL  2

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
{
   int i;
   int totalOrders;
   int ticket;
   double tempOrder;
   int order = SIGNAL_NONE;

   double lsr0 = iCustom(Symbol(),0,"LevelStop-Reverse2.0",UseATRMode,NonATRStopPips,ATRPeriod,ATRMultiplier,ATRSmoothing,UpArrowColor,DnArrowColor,ArrowDistance,AlertSound,AlertMail,ShowComment,0,0);
   double lsr1 = iCustom(Symbol(),0,"LevelStop-Reverse2.0",UseATRMode,NonATRStopPips,ATRPeriod,ATRMultiplier,ATRSmoothing,UpArrowColor,DnArrowColor,ArrowDistance,AlertSound,AlertMail,ShowComment,0,1);
   double lsr2 = iCustom(Symbol(),0,"LevelStop-Reverse2.0",UseATRMode,NonATRStopPips,ATRPeriod,ATRMultiplier,ATRSmoothing,UpArrowColor,DnArrowColor,ArrowDistance,AlertSound,AlertMail,ShowComment,0,2);
   lsr0 = NormalizeDouble(lsr0,Digits);
   lsr1 = NormalizeDouble(lsr1,Digits);
   lsr2 = NormalizeDouble(lsr2,Digits);
   
   if (Close [1] > lsr1 && Close [2] < lsr2)
   {
      order = SIGNAL_BUY;
   }
   else if (Close [1] < lsr1 && Close [2] > lsr2)
   {
      order = SIGNAL_SELL;
   }
   else
   {
      order = SIGNAL_NONE;
   }
   
   if (DumbTradeRules)
   {
      // Check current positions to see if they need to be closed
      // due to the stop flopping over.
      totalOrders=OrdersTotal();
      for (i=0;i<totalOrders;i++)
      {
         OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
         if (OrderSymbol() == Symbol())
         {
            // If the stop has flopped to the other side of the trade then close it.
            if (OrderType() == OP_BUY && order == SIGNAL_SELL)
            {
               Print("closing long position");
               OrderClose (OrderTicket(),Lots,Bid,3);
            }
            
            if (OrderType() == OP_SELL && order == SIGNAL_BUY)
            {
               Print("closing short position");
               OrderClose (OrderTicket(),Lots,Ask,3);
            }
         } // end if symbol
      } // end for i
      
      // If there aren't any open positions and it's the right time of day,
      // then open one in the right direction.
      totalOrders=OrdersTotal();
      if (totalOrders == 0 && ((Hour()>=FromHourTrade && Hour()<=ToHourTrade) || !UseHourTrade))
      {
         // If the current signal is buy, then go long
         if (order == SIGNAL_BUY)
         {
            Print ("Creating buy order");
            ticket=OrderSend(Symbol(),OP_BUY,Lots,Ask,3,0,0,"initial buy",16384,0,Green);
            if(ticket>0)
            {
               if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
               {
                  Print("BUY order opened : ",OrderOpenPrice());
               }
            }
            else
            {
               Print("Error opening BUY order : ",GetLastError()); 
            }
         }
         // If the current signal is sell, then go short
         else if (order == SIGNAL_SELL)
         {
            Print ("Creating sell order");
            ticket=OrderSend(Symbol(),OP_SELL,Lots,Bid,3,0,0,"initial sell",16384,0,Red);
            if(ticket>0)
            {
               if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES))
               {
                  Print("SELL order opened : ",OrderOpenPrice());
               }
            }
            else
            {
               Print("Error opening SELL order : ",GetLastError()); 
            }
         }
      } // end else totalorders > 0
   } // End if dumb trade rules
   else
   {
   } // End else dumb trade rules
   
   return(0);
}

